// File:          RopeController.cpp
// Date:
// Description:
// Author:
// Modifications:

// You may need to add webots include files such as
// <webots/DistanceSensor.hpp>, <webots/LED.hpp>, etc.
// and/or to add some other includes

#include <vector>
#include <string>
#include <iostream>

#include <libwebots/communication/client.hh>
#include <libwebots/plugin/modules/jointmanager/jointmanagerbuilder.hh>
#include <libwebots/plugin/modules/debug/debugbuilder.hh>

#include <webots/Supervisor.hpp>

using namespace libwebots::communication;
using namespace libwebots::plugin::modules::messages;


struct LinkDefinition {
	std::string parent;
	std::string child;
	double length;

	LinkDefinition(const std::string & parent_,
	               const std::string & child_,
	               double length_): parent(parent_),
	               child(child_),
	               length(length_){};
};

int main(int argc, char **argv)
{

	webots::Supervisor * supervisor = new webots::Supervisor();
	Client & client = Client::Instance();

	client.Connect();

	client.Send(builders::Debug::Enable(debug::Type::Info));

	//stiffness damping params, O : angle, 1 : torsion
	Eigen::ArrayXd stiffness(Eigen::ArrayXd::Ones(2) * 0.5);
	Eigen::ArrayXd damping(Eigen::ArrayXd::Ones(2) * 0.1);


	//just a compact form for all parameters that we need to set;
	std::vector<LinkDefinition> bodyChain;
	bodyChain.push_back(LinkDefinition("SUPPORT","ROPE_1",0.025));
	bodyChain.push_back(LinkDefinition("ROPE_1","ROPE_2",0.025));
	bodyChain.push_back(LinkDefinition("ROPE_2","ROPE_3",0.025));
	bodyChain.push_back(LinkDefinition("ROPE_3","ROPE_4",0.025));
	bodyChain.push_back(LinkDefinition("ROPE_4","ROPE_5",0.025));
	bodyChain.push_back(LinkDefinition("ROPE_5","BALL_MASS",0.03));

	Eigen::Vector3d axis(Eigen::Vector3d::Zero());
	axis[1] = 1.0;
	unsigned int uid = 0;
	Eigen::Vector3d position;
	position << 0,0,0;
	for(std::vector<LinkDefinition>::const_iterator ld = bodyChain.begin();
			ld != bodyChain.end();
			++ld){
		uid = ld - bodyChain.begin();
		position[1] = ld->length;
    std::cout << "Processing " << ld->parent << " " << ld->child << std::endl;
		//First we create the joint
		Message m = builders::JointManager::CreateBallJoint(uid,
		                                                    ld->parent,
		                                                    ld->child,
		                                                    jointmanager::JointDefinition::CHILD,
		                                                    position,
		                                                    axis,
		                                                    0.1);

		client.Send(m);


		//now we set some joint limit;
		builders::JointManager::JointParameterValue *pv;
		builders::JointManager::ListOfParameterValue pvList;
		pvList.Reserve(4);
		pv = pvList.Add();
		pv->set_parameter(jointmanager::JointParameters::dParamLoStop);
		pv->set_value(-0.001);
		pv->set_dimension(0);

		pv = pvList.Add();
		pv->set_parameter(jointmanager::JointParameters::dParamHiStop);
		pv->set_value(1.4);
		pv->set_dimension(0);

		pv = pvList.Add();
		pv->set_parameter(jointmanager::JointParameters::dParamLoStop);
		pv->set_value(-0.001);
		pv->set_dimension(1);

		pv = pvList.Add();
		pv->set_parameter(jointmanager::JointParameters::dParamHiStop);
		pv->set_value(1.4);
		pv->set_dimension(1);

		client.Send(builders::JointManager::SetJointParameters(uid,pvList));
		//now adds joint limits, Ball joint are 3D joint, but only 2D controllable

		client.Send(builders::JointManager::SetJointPassiveDynamic(uid,stiffness,damping,Eigen::ArrayXd::Zero(2)));
	}
	while(true){
		supervisor->step(20);
	}
}
