#ifndef __LIBWEBOTS_COMMUNICATION_CLIENT_H__
#define __LIBWEBOTS_COMMUNICATION_CLIENT_H__

#include <google/protobuf/message.h>
#include <string>
#include <sstream>
#include <pthread.h>

#include <libwebots/communication/socket.hh>
#include <libwebots/communication/handlers.hh>
#include <libwebots/communication/message.pb.h>

namespace libwebots
{
namespace communication
{
	class Client : public Socket, public Handlers
	{
		static Client *s_instance;

		struct State
		{
			enum Values
			{
				Closed,
				Connecting,
				Connected,
				Closing
			};
		};

		std::string d_data;
		State::Values d_state;

		bool d_async;

		pthread_t d_rwThread;
		pthread_mutex_t d_rwLock;

		public:
			static Client &Instance();
			static Client &Initialize(bool async = false);

			static void Finalize();

			Client(bool async = false);
			Client(int fd, bool async = false);

			void Connect();

			bool Send(Message const &message);
			bool Receive(Message &message);

			bool Connecting();
			bool Connected();
			bool Closing();
			bool Closed();

			bool ProcessMessages();

			virtual void Close();
		private:
			void ReadData();
			bool FinishConnect();
			bool IsState(State::Values state);

			void ConnectDone();

			static void *ReadWriteThread(void *client);
	};
}
}

#endif /* __LIBWEBOTS_COMMUNICATION_CLIENT_H__ */

