#include "handler.hh"

using namespace libwebots::communication;
using namespace std;

/**
 * @class libwebots::communication::Handler
 * @brief Base callback handler
 *
 * Base class for template callback handlers. This base class stores the message
 * type id and the optional filter for the callback. The object and member
 * callback are registered in specific templated classes (see HandlerCallback)
 *
 **/

/**
 * @brief Create new Handler
 * @param id the handler id
 * @param match the handler match
 *
 * Create a new handler
 *
 **/
Handler::Handler(int id, string const &match)
:
	d_id(id),
	d_match(match)
{
}


Handler::~Handler(){
}

/**
 * @brief Get the handler id
 *
 * Get the handler id
 *
 * @returns the handler id
 *
 **/
int
Handler::Id() const
{
	return d_id;
}

/**
 * @brief Get the handler match
 *
 * Get the handler match
 *
 * @returns the handler match
 *
 **/
string const &
Handler::Match() const
{
	return d_match;
}
