#ifndef __LIBWEBOTS_PLUGIN_BODY_H__
#define __LIBWEBOTS_PLUGIN_BODY_H__

#include <map>
#include <vector>
#include <plugins/physics.h>
#include <Eigen/Core>
#include <Eigen/Geometry>

namespace libwebots
{
namespace plugin
{
	class Body : public std::map<dJointID, Body>
	{
		dBodyID d_id;
		bool d_iterated;
		std::string d_def;

		Eigen::Matrix3d d_localRotation;
		Eigen::Vector3d d_localTranslation;

		typedef std::map<dJointID, Body> BaseType;

		public:
			Body(std::string const &def);
			Body(dBodyID id = 0);
			Body(Body const &other);
			Body &operator=(Body const &other);

			dBodyID Id() const;

			operator bool() const;

			bool Includes(dBodyID body) const;
			bool Includes(Body const &other) const;

			Eigen::Vector3d AngularVelocity() const;
			Eigen::Vector3d LinearVelocity() const;
			Eigen::Vector3d Translation() const;
			Eigen::Matrix3d Rotation() const;
			Eigen::Quaternion<double> Quaternion() const;

			Eigen::Vector3d const &LocalTranslation() const;
			Eigen::Matrix3d const &LocalRotation() const;

			void SetAngularVelocity(Eigen::Vector3d const &angvel);
			void SetLinearVelocity(Eigen::Vector3d const &linvel);
			void SetQuaternion(Eigen::Quaternion<double> const &quat);
			void SetTranslation(Eigen::Vector3d const &pos);
			void SetRotation(Eigen::Matrix3d const &rotation);

			iterator begin();
			iterator end();
			const_iterator begin() const;
			const_iterator end() const;

			std::vector<Body> Descendents(bool includeSelf = false);
			std::vector<dGeomID> Geoms() const;
		private:
			void LocalTransform();
			void FindChildren();
			void Descendents(std::vector<Body> &bodies, bool includeSelf);
	};
}
}

#endif /* __LIBWEBOTS_PLUGIN_BODY_H__ */
