#ifndef __LIBWEBOTS_PLUGIN_MODULE_H__
#define __LIBWEBOTS_PLUGIN_MODULE_H__

#include <map>
#include <libwebots/plugin/typemap.hh>
#include <libwebots/plugin/interface.hh>
#include <iostream>

namespace libwebots
{
namespace plugin
{
	class Module
	{
		public:
			/**
			 * @brief Map from TypeID to module instance
			 *
			 * A map from a TypeID to a Module instance
			 *
			 **/
			typedef std::map<TypeID, Module *> MapByTypeID;

			Module(MapByTypeID const &registeredDependencies);
			virtual ~Module();

			static void RegisterDependencies();
		protected:
			template <typename ModuleName>
			modules::interfaces::Interface<ModuleName> GetInterface();
		private:
			Module::MapByTypeID d_registeredDependencies;
	};

	/**
	 * @brief Get registered dependency interface
	 *
	 * Get the interface of a registered dependency (registered in
	 * RegisterDependencies()). You can only retrieve interfaces for
	 * modules which are explicitly registered (and only for modules that
	 * provide such an interface, e.g. libwebots::plugin::modules::Debug module).
	 *
	 **/
	template <typename ModuleName>
	modules::interfaces::Interface<ModuleName> Module::GetInterface()
	{
		MapByTypeID::iterator fi = d_registeredDependencies.find(TypeMap<ModuleName>::ID());

		if (fi == d_registeredDependencies.end())
		{
			std::cerr << "You tried to access the interface of a dependency not "
			          << "listed in your module. This is uterly insane. You get "
			          << "punishement and can restart Webots!"
			          << std::endl;

			int *unregisteredDependency = 0;
			*unregisteredDependency = 42;
		}

		return modules::interfaces::Interface<ModuleName>(*dynamic_cast<ModuleName *>(fi->second));
	}
}
}

#endif /* __LIBWEBOTS_PLUGIN_MODULE_H__ */

