#ifndef __LIBWEBOTS_PLUGIN_MODULES_CONSTRAINT_H__
#define __LIBWEBOTS_PLUGIN_MODULES_CONSTRAINT_H__

#include <libwebots/plugin/physicsmodule.hh>
#include <libwebots/plugin/modules/constraint/constraint.pb.h>
#include <libwebots/plugin/modules/debug/debug.hh>

#include <map>

namespace libwebots
{
namespace plugin
{
namespace modules
{
	class Constraint : public PhysicsModule, public interfaces::Debugable
	{
		struct Item
		{
			std::string Name;
			dBodyID Body;
			dJointID Joint;

			bool DestroyJoint;

			double Timeout;
			bool DestroyOnTimeout;
		};

		struct Monitor
		{
			std::string Name;

			dJointID Joint;
			dJointFeedback JointFeedback;

			double PreviousVelocity;
		};

		std::vector<Item *> d_timeouts;
		std::vector<Item *> d_constraints;
		std::map<std::string, Monitor *> d_monitors;

		double d_previousTime;

//		interfaces::Interface<Debug> d_debug;

		public:
			Constraint(dWorldID world,
			           dSpaceID space,
			           dJointGroupID jointGroup,
			           MapByTypeID const &dependencies);

			~Constraint();

			virtual void Step();
			virtual void StepEnd();

			void AddMonitor(std::string const &name);
		private:
			void Handler(messages::constraint::Constraint const &message);

			void Handler(messages::constraint::FixInWorld const &message);
			void Handler(messages::constraint::FixInPlane const &message);
			void Handler(messages::constraint::FixInAxis const &message);
			void Handler(messages::constraint::Destroy const &message);
			void Handler(messages::constraint::Enable const &message);
			void Handler(messages::constraint::MonitorJoint const &message);
			void Handler(messages::constraint::SetVelocity const &message);

			bool FindBody(std::string const &name, dBodyID &body);

			void Add(std::string const &name,
			         dBodyID            body,
			         dJointID           joint,
			         double             timeout,
			         bool               destroyOnTimeout,
			         bool               destroyJoint,
			         bool               monitor);

			void Destroy(Item *item, bool remove);
			void Enable(Item *item, bool enable);

			void AddMonitor(std::string const &name,
			                dJointID           joint);
	};
}
}
}

#endif /* __LIBWEBOTS_PLUGIN_MODULES_CONSTRAINT_H__ */

