#include "constraintbuilder.hh"
#include "constraint.pb.h"

using namespace libwebots::plugin::modules::messages;
using namespace libwebots::communication;
using namespace std;

Message
builders::Constraint::FixInWorld(string const &body, bool monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInWorld);

	constraint::FixInWorld &msg = *cons.mutable_fixinworld();

	msg.set_body(body);
	msg.set_monitor(monitor);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInAxis(string const &body, bool monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInAxis);

	constraint::FixInAxis &msg = *cons.mutable_fixinaxis();

	msg.set_body(body);
	msg.set_monitor(monitor);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInAxis(string const &body, double axis[3], bool monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInAxis);

	constraint::FixInAxis &msg = *cons.mutable_fixinaxis();

	msg.set_body(body);
	msg.set_monitor(monitor);

	msg.add_axis(axis[0]);
	msg.add_axis(axis[1]);
	msg.add_axis(axis[2]);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                                 bool          canRotate,
                                 bool          monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);

	msg.set_monitor1(monitor);
	msg.set_monitor2(monitor);

	msg.set_canrotate(canRotate);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                                 double        rotationMinStop,
                                 double        rotationMaxStop,
                                 bool          monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);

	msg.set_monitor1(monitor);
	msg.set_monitor2(monitor);

	msg.set_canrotate(true);
	msg.add_rotation_stops(rotationMinStop);
	msg.add_rotation_stops(rotationMaxStop);
	msg.set_monitorrotation(monitor);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                                 double        axis1[3],
                                 bool          canRotate,
                                 bool          monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);
	msg.set_monitor1(monitor);
	msg.set_monitor2(monitor);

	for (size_t i = 0; i < 3; ++i)
	{
		msg.add_axis1(axis1[i]);
	}

	msg.set_canrotate(canRotate);

	if (canRotate)
	{
		msg.set_monitorrotation(monitor);
	}

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                                 double        axis1[3],
                                 double        rotationMinStop,
                                 double        rotationMaxStop,
                                 bool          monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);
	msg.set_monitor1(monitor);
	msg.set_monitor2(monitor);

	for (size_t i = 0; i < 3; ++i)
	{
		msg.add_axis1(axis1[i]);
	}

	msg.set_canrotate(true);
	msg.add_rotation_stops(rotationMinStop);
	msg.add_rotation_stops(rotationMaxStop);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                                 double        axis1[3],
                                 double        axis2[3],
                                 bool          canRotate,
                                 bool          monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);
	msg.set_monitor1(monitor);
	msg.set_monitor2(monitor);

	for (size_t i = 0; i < 3; ++i)
	{
		msg.add_axis1(axis1[i]);
		msg.add_axis2(axis2[i]);
	}

	msg.set_canrotate(canRotate);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                                 double        axis1[3],
                                 double        axis2[3],
                                 double        rotationMinStop,
                                 double        rotationMaxStop,
                                 bool          monitor)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);
	msg.set_monitor1(monitor);
	msg.set_monitor2(monitor);

	for (size_t i = 0; i < 3; ++i)
	{
		msg.add_axis1(axis1[i]);
		msg.add_axis2(axis2[i]);
	}

	msg.set_canrotate(true);
	msg.add_rotation_stops(rotationMinStop);
	msg.add_rotation_stops(rotationMaxStop);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                              double           axis1[3],
                              double           axis2[3],
                              double           disable1Timeout,
                              double           disable2Timeout,
                              double           disableRotationTimeout,
                              bool             destroy1OnTimeout,
                              bool             destroy2OnTimeout,
                              bool             destroyRotationOnTimeout,
                              bool             canRotate,
                              bool             monitor1,
                              bool             monitor2,
                              bool             monitorRotation)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);

	for (size_t i = 0; i < 3; ++i)
	{
		msg.add_axis1(axis1[i]);
		msg.add_axis2(axis2[i]);
	}

	msg.set_disable1_timeout(disable1Timeout);
	msg.set_disable2_timeout(disable2Timeout);
	msg.set_destroy1_on_timeout(destroy1OnTimeout);
	msg.set_destroy2_on_timeout(destroy2OnTimeout);

	msg.set_monitor1(monitor1);
	msg.set_monitor2(monitor2);

	msg.set_canrotate(canRotate);
	msg.set_monitorrotation(monitorRotation);
	msg.set_disablerotation_timeout(disableRotationTimeout);
	msg.set_destroyrotation_on_timeout(destroyRotationOnTimeout);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::FixInPlane(string const &body,
                              double           axis1[3],
                              double           axis2[3],
                              double           disable1Timeout,
                              double           disable2Timeout,
                              double           disableRotationTimeout,
                              bool             destroy1OnTimeout,
                              bool             destroy2OnTimeout,
                              bool             destroyRotationOnTimeout,
                              double           rotationMinStop,
                              double           rotationMaxStop,
                              bool             monitor1,
                              bool             monitor2,
                              bool             monitorRotation)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::FixInPlane &msg = *cons.mutable_fixinplane();

	msg.set_body(body);

	for (size_t i = 0; i < 3; ++i)
	{
		msg.add_axis1(axis1[i]);
		msg.add_axis2(axis2[i]);
	}

	msg.set_disable1_timeout(disable1Timeout);
	msg.set_disable2_timeout(disable2Timeout);
	msg.set_destroy1_on_timeout(destroy1OnTimeout);
	msg.set_destroy2_on_timeout(destroy2OnTimeout);

	msg.set_monitor1(monitor1);
	msg.set_monitor2(monitor2);

	msg.set_canrotate(true);
	msg.add_rotation_stops(rotationMinStop);
	msg.add_rotation_stops(rotationMaxStop);

	msg.set_monitorrotation(monitorRotation);
	msg.set_disablerotation_timeout(disableRotationTimeout);
	msg.set_destroyrotation_on_timeout(destroyRotationOnTimeout);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::Destroy(string const &body)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::Destroy);

	constraint::Destroy &msg = *cons.mutable_destroy();
	msg.set_body(body);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::Enable(string const &body, bool enable)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::FixInPlane);

	constraint::Enable &msg = *cons.mutable_enable();

	msg.set_body(body);
	msg.set_enable(enable);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::SetVelocity(string const &body, double velocity)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::SetVelocity);

	constraint::SetVelocity &msg = *cons.mutable_setvelocity();

	msg.set_body(body);
	msg.set_velocity(velocity);

	return Messages::Create(constraint::constraint, cons);
}

Message
builders::Constraint::MonitorJoint(string const &body)
{
	constraint::Constraint cons;

	cons.set_action(constraint::Action::MonitorJoint);

	constraint::MonitorJoint &msg = *cons.mutable_monitorjoint();
	msg.set_body(body);

	return Messages::Create(constraint::constraint, cons);
}
