#ifndef __LIBWEBOTS_PLUGIN_MODULES_CONTACT_H__
#define __LIBWEBOTS_PLUGIN_MODULES_CONTACT_H__

#include <libwebots/plugin/physicsmodule.hh>
#include <libwebots/plugin/modules/contact/contact.pb.h>
#include <libwebots/plugin/modules/debug/debug.hh>

#include <map>

namespace libwebots
{
namespace plugin
{
namespace modules
{
	class Contact : public PhysicsModule , public interfaces::Debugable
	{
		struct MaxContacts
		{
			enum
			{
				Value = 16
			};
		};

		struct Monitor
		{
			std::string Name1;
			std::string Name2;

			dGeomID Geom1;
			dGeomID Geom2;

			int Mode;
			bool CustomModel;
			bool Monitor;

			double Bounce;
			double BounceVelocity;
			double CoulombFriction;
			double ForceDependentSlip;
			double SoftERP;
			double SoftCFM;

			bool DrawContact;

			dJointFeedback JointFeedback[MaxContacts::Value];
			dContact Contacts[MaxContacts::Value];

			size_t NumContacts;
		};

		std::vector<Monitor *> d_monitors;
		std::multimap<dGeomID, Monitor *> d_monitorMap;
		std::vector<Monitor *> d_spaces;

		public:
			Contact(dWorldID world,
			        dSpaceID space,
			        dJointGroupID jointGroup,
			        Module::MapByTypeID const &dependencies);

			~Contact();

			virtual bool Collide(dGeomID g1, dGeomID g2);
			virtual void StepEnd();
			virtual void Draw();
		private:
			void Handler(messages::contact::Contact const &message);
			bool FindGeom(std::string const &name, dGeomID &geom);

			bool SpaceMatch(dGeomID space, dGeomID g1);

			Monitor *MatchMonitor(dGeomID g1, dGeomID g2);
			void SetupSurface(Monitor *monitor, size_t i);
			void ComputeDirection(Monitor *monitor, size_t i);

			void DrawContact(dContact const &contact, dJointFeedback const &feedback, bool vec);
	};
}
}
}

#endif /* __LIBWEBOTS_PLUGIN_MODULES_CONTACT_H__ */

