#ifndef __LIBWEBOTS_PLUGIN_MODULES_DEBUG_H__
#define __LIBWEBOTS_PLUGIN_MODULES_DEBUG_H__

#include <libwebots/plugin/module.hh>
#include <libwebots/plugin/modules/debug/debug.pb.h>

#include <sstream>
#include <iomanip>

namespace libwebots
{
namespace plugin
{
namespace modules
{
	class Debug : public Module
	{
		static messages::debug::Type::Values s_defaultType;

		messages::debug::Type::Values d_enabled;
		messages::debug::Redirection::Values d_redirection;

		public:
			Debug(Module::MapByTypeID const &dependencies);

			void Send(messages::debug::Type::Values type, std::string const &message);

			static void RegisterDependencies();

			static void SetDefaultType(messages::debug::Type::Values type);
		private:
			void Handler(messages::debug::Debug const &message);
	};

	namespace interfaces
	{
		template <>
		class Interface<Debug> : public std::stringbuf
		{
			Debug &d_debug;
			messages::debug::Type::Values d_type;

			std::ostream d_buffer;

			public:
				Interface(Debug &module);
				Interface(Interface<Debug> const &other);

				Interface<Debug> &operator=(Interface<Debug> const &other);

				std::ostream &operator()(messages::debug::Type::Values type
				                         = messages::debug::Type::Error);
			protected:
				virtual int sync();
		};

		class Debugable {
		public :
			Debugable(const Interface<Debug> & debug,
			          const std::string & prefix);
		protected :
			std::ostream & Info(bool withPrefix = true);
			std::ostream & Warning(bool withPrefix = true);
			std::ostream & Error(bool withPrefix = true);

		private :
			std::ostream & GetOut(messages::debug::Type::Values type,
			                      bool withPrefix);

			interfaces::Interface<Debug>   d_debug;
			std::string        d_prefix;
		};

		inline std::ostream &
		Debugable::GetOut(messages::debug::Type::Values type,
		                  bool withPrefix){
			std::ostream & out = d_debug(type);

			if(withPrefix && !d_prefix.empty()){
				out << d_prefix;
			}
			return out;
		}

		inline std::ostream &
		Debugable::Info(bool withPrefix){
			return GetOut(messages::debug::Type::Info,withPrefix);
		}

		inline std::ostream &
		Debugable::Warning(bool withPrefix){
			return GetOut(messages::debug::Type::Warning,withPrefix);
		}

		inline std::ostream &
		Debugable::Error(bool withPrefix){
			return GetOut(messages::debug::Type::Error,withPrefix);
		}

	}
}
}
}

#endif /* __LIBWEBOTS_PLUGIN_MODULES_DEBUG_H__ */

