/**
 * \file Joint.h
 *
 *  \date Nov 23, 2011
 *  \author Alexandre Tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_MODULES_JOINTCREATOR_JOINT_HH_
#define __LIBWEBOTS_PLUGIN_MODULES_JOINTCREATOR_JOINT_HH_

#include <libwebots/plugin/modules/jointmanager/jointmanager.pb.h>
#include <Eigen/Eigen>
#include <plugins/physics.h>
#include <memory>
#include <libwebots/plugin/body.hh>
namespace libwebots
{
namespace plugin
{
namespace modules
{
namespace jointmanager
{
	class Joint
	{
	public:
		static void RegisterFactories();
//		static dJointType
//		TypeFromMessage(messages::jointmanager::JointDefinition::Type type);

		static int
		ParameterFromMessage(messages::jointmanager::JointParameters::Values parameter);

		static
		std::auto_ptr<Joint>
		Create(messages::jointmanager::JointDefinition::Type type);

		static
		std::auto_ptr<Joint>
		Create(const Body & parent, const Body & child);

		virtual ~Joint();

		virtual void Step(double ts);

		virtual unsigned int Dimension() const = 0;

		virtual void SetAnchor(const dVector3 worldPosition) = 0;
		virtual void SetAxis(const dVector3 worldAxis) = 0;

		void SetAnchor(const Eigen::Vector3d & worldPosition);
		void SetAxis(const Eigen::Vector3d & worldPosition);

		virtual void SetParameter(int parameter,
		                          double value,
		                          unsigned int dim = 0) = 0;

		operator bool() const;

		virtual std::auto_ptr<Joint> Clone() const = 0;

		virtual void AddEffort(const Eigen::Vector3d & value) = 0;

		const Eigen::Vector3d & GetCoordinateConst() const;
		const Eigen::Vector3d & GetCoordinateRateConst() const;

		virtual void Draw(double size)       = 0;

		void Create(dWorldID world, dJointGroupID group);

		void Attach(const Body & parent, const Body & child);
		virtual void Attach(dBodyID parent, dBodyID child);

		dJointID Id() const;
		dBodyID  Parent() const;
		dBodyID  Child() const;
	protected :
		Joint();
		virtual void ComputeCoordinateAndRate(double ts)= 0;

		Eigen::Vector3d & GetCoordinate();
		Eigen::Vector3d & GetCoordinateRate();


		virtual dJointID CreatePrimpl(dWorldID world, dJointGroupID group) = 0;
	public :
		EIGEN_MAKE_ALIGNED_OPERATOR_NEW_IF_VECTORIZABLE_FIXED_SIZE(double,3);
	private :

		typedef messages::jointmanager::JointDefinition::Type JointType;
		typedef messages::jointmanager::JointParameters::Values JointParameter;
		typedef std::map<JointType,Joint*> JointFactory;
		typedef std::map<dJointType,Joint*> JointODEFactory;
		typedef std::map<JointParameter,int> ParameterFactory;

		void SetID(dJointID id);
		void SetManaged(bool managed = true);

		static JointFactory     s_factory;
		static JointODEFactory  s_odeFactory;
		static ParameterFactory s_parameters;

		dJointID d_id;
		bool d_managed;
		Eigen::Vector3d d_coordinate;
		Eigen::Vector3d d_rate;

};

} //namespace jointmanager
} //namespace modules
} //namespace plugin
} //namespace libwebots

#endif // __LIBWEBOTS_PLUGIN_MODULES_JOINTCREATOR_JOINT_HH_
