/**
 * \file jointmanager.hh
 *
 *  \date Nov 18, 2011
 *  \author Alexandre Tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_MODULES_JOINTMANAGER_HH_
#define __LIBWEBOTS_PLUGIN_MODULES_JOINTMANAGER_HH_

#include <libwebots/plugin/physicsmodule.hh>
#include <libwebots/plugin/modules/jointmanager/jointmanager.pb.h>
#include <libwebots/plugin/modules/debug/debug.hh>
#include <tr1/memory>
#include <libwebots/plugin/modules/jointmanager/joints/joints.hh>
#include <libwebots/plugin/modules/jointmanager/jointmonitor.hh>

namespace libwebots
{
namespace plugin
{
namespace modules
{
	namespace jointmanager
	{
		class PassiveDynamics;
	}
	class JointManager : public PhysicsModule, public interfaces::Debugable
	{
		public :
			static void RegisterDependencies();


			JointManager(dWorldID world,
			             dSpaceID space,
			             dJointGroupID jointGroup,
			             Module::MapByTypeID const & dependencies);

			virtual ~JointManager();

			virtual void Step();
			virtual void Draw();
			virtual void StepEnd();

		private :
			typedef uint32_t JointIdentifier;
			typedef std::tr1::shared_ptr<jointmanager::Joint> JointPtr;
			typedef std::tr1::shared_ptr<jointmanager::PassiveDynamics> PassiveDynamicsPtr;
			typedef std::tr1::shared_ptr<jointmanager::JointMonitor>    MonitorPtr;

			typedef std::map<JointIdentifier,JointPtr>           JointByIdentifier;
			typedef std::map<JointPtr,double>                    SizeByJoint;
			typedef std::map<JointIdentifier,PassiveDynamicsPtr> PassiveDynamicsByIdentifier;
			typedef std::map<JointIdentifier,MonitorPtr>         MonitorByIdentifier;

			typedef ::google::protobuf::RepeatedPtrField<messages::jointmanager::SetJointParameter::ParameterValue> ListOfParameterValue;

			void Handler(messages::jointmanager::JointManager const & message);

			void
			CreateHandler(messages::jointmanager::Create const & message);

			void
			DestroyHandler(messages::jointmanager::Destroy const & message);

			void
			ManageHandler(messages::jointmanager::AddManagedJoint const & message);

			void
			SetParameterHandler(messages::jointmanager::SetJointParameter const & message);

			void
			SetPassiveDynamicsHandler(messages::jointmanager::SetPassiveDynamics const & message);

			void
			AddMonitorHandler(messages::jointmanager::AddMonitor const & message);

			void
			DeleteMonitorHandler(messages::jointmanager::DeleteMonitor const & message);


			JointByIdentifier            d_joints;
			PassiveDynamicsByIdentifier  d_passiveDynamics;
			MonitorByIdentifier          d_monitors;
			SizeByJoint                  d_drawSizes;
//			bool                         d_displayAxis;
	};
}
}
}


#endif // __LIBWEBOTS_PLUGIN_MODULES_JOINTMANAGER_HH_
