/**
 * \file jointmanagerbuilder.hh
 *
 *  \date Nov 18, 2011
 *  \author Alexandre Tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_MODULES_MESSAGES_BUILDER_JOINTMANAGER_HH_
#define __LIBWEBOTS_PLUGIN_MODULES_MESSAGES_BUILDER_JOINTMANAGER_HH_

#include <libwebots/communication/messages.hh>
#include <libwebots/plugin/modules/jointmanager/jointmanager.pb.h>
#include <Eigen/Eigen>

namespace libwebots
{
namespace plugin
{
namespace modules
{
namespace messages
{
namespace builders
{

	class JointManager
	{
		public :
			typedef uint32_t JointIdentifier;
			typedef messages::jointmanager::SetJointParameter::ParameterValue JointParameterValue;
			typedef ::google::protobuf::RepeatedPtrField<messages::jointmanager::SetJointParameter::ParameterValue> ListOfParameterValue;

			static communication::Message
				CreateHingeJoint(JointIdentifier identifier,
				                 std::string const & parent,
				                 std::string const & child,
				                 jointmanager::JointDefinition::Frame frame,
				                 const Eigen::Vector3d & position,
				                 const Eigen::Vector3d & axis,
				                 double drawSize = 0.0);
			static communication::Message
				CreateSliderJoint(JointIdentifier identifier,
				                  std::string const & parent,
				                  std::string const & child,
				                  jointmanager::JointDefinition::Frame frame,
				                  const Eigen::Vector3d & axis);

			static communication::Message
				CreateFixedJoint(JointIdentifier identifier,
				                 std::string const & parent,
				                 std::string const & child);

			static communication::Message
				CreateBallJoint(JointIdentifier identifier,
						        std::string const & parent,
						        std::string const & child,
						        jointmanager::JointDefinition::Frame frame,
						        const Eigen::Vector3d & position,
						        const Eigen::Vector3d & axis,
						        double drawSize = 0.0);

			static communication::Message
				DestroyJoint(JointIdentifier identifier);

			static communication::Message
				AddManagedjoint(JointIdentifier identifier,
				                const std::string & parent,
				                const std::string & child);

			static communication::Message
				SetJointParameter(JointIdentifier identifier,
				                  messages::jointmanager::JointParameters::Values param,
				                  double value,
				                  uint32_t dimension = 0);

			static communication::Message
				SetJointParameters(JointIdentifier identifier,
				                   const ListOfParameterValue & listOfParameters);

			static communication::Message
				SetJointPassiveDynamic(JointIdentifier identifier,
				                       double stiffness,
				                       double damping,
				                       double precompression);

			static communication::Message
				SetJointPassiveDynamic(JointIdentifier identifier,
				                       const Eigen::ArrayXd & stiffness,
				                       const Eigen::ArrayXd & damping,
				                       const Eigen::ArrayXd & precompression);

			static communication::Message
				SetJointPassiveDynamicsReferencePosition(JointIdentifier identifier,
				                                         double referencePosition);

			static communication::Message
				AddMonitor(JointIdentifier id,
				           jointmanager::JointDefinition::Frame frame,
				           jointmanager::AddMonitor::Type type);

			static communication::Message
				DeleteMonitor(JointIdentifier id);

	};

}
}
}
}
}

#endif // __LIBWEBOTS_PLUGIN_MODULES_MESSAGES_BUILDER_KINEMATICLOOP_HH_
