/**
 * \file jointmonitor.cc
 *
 *  \date Jan 24, 2013
 *  \author tuleu
 */

#include "jointmonitor.hh"
#include "joints/joints.hh"

namespace libwebots
{
namespace plugin {

namespace modules {

namespace jointmanager {

JointMonitor::JointMonitor(const Joint & joint,
                           Frame frame)
: d_joint(joint)
, d_frame(frame){}

JointMonitor::~JointMonitor(){}

void JointMonitor::PrepareMessage(messages::jointmanager::Monitor & m){
	dVector3 force,torque;
	GetForceAndTorque(force,torque);
	if(d_frame == JD::PARENT){
		ToBody(d_joint.Parent(),force,torque);
	} else if (d_frame == JD::CHILD){
		ToBody(d_joint.Child(),force,torque);
	}

	m.set_frame(d_frame);

	m.mutable_force()->set_x(force[0]);
	m.mutable_force()->set_y(force[1]);
	m.mutable_force()->set_z(force[2]);

	m.mutable_torque()->set_x(torque[0]);
	m.mutable_torque()->set_y(torque[1]);
	m.mutable_torque()->set_z(torque[2]);
}

const Joint & JointMonitor::GetJoint() const{
	return d_joint;
}

void JointMonitor::ToBody(dBodyID b,
                          dVector3 & force,
                          dVector3 & torque){
	dBodyVectorFromWorld(b,force[0],force[1],force[2],force);
	dBodyVectorFromWorld(b,torque[0],torque[1],torque[2],torque);
}


JointMonitorByDifference::JointMonitorByDifference(const Joint & joint,
                                                   Frame frame)
	: JointMonitor(joint,frame){
}

JointMonitorByDifference::~JointMonitorByDifference(){
}

void JointMonitorByDifference::GetForceAndTorque(dVector3 & force , dVector3 & torque){
//	const dReal * pForce = dBodyGetForce(dJointGetBody(GetJoint().Id(),0));
//	const dReal * cForce = dBodyGetForce(dJointGetBody(GetJoint().Id(),1));

	const dReal * pTorque = dBodyGetTorque(dJointGetBody(GetJoint().Id(),0));
	const dReal * cTorque = dBodyGetTorque(dJointGetBody(GetJoint().Id(),1));

	for(unsigned int i = 0; i < 3; ++i){
		torque[i] = cTorque[i] - pTorque[i];
		force[i]  = 0;
	}

	//force is more complex, we assume it is applied at CoM, we should translate
	//it to joint pos
	//\todo implement
}


JointMonitorByFeedback::JointMonitorByFeedback(const Joint & joint,
                                               Frame frame)
	: JointMonitor(joint,frame){
	for(unsigned int i = 0; i < 3; ++i){
		d_feedback.f1[i] = 0;
		d_feedback.f2[i] = 0;
		d_feedback.t1[i] = 0;
		d_feedback.t2[i] = 0;
	}
	dJointSetFeedback(joint.Id(),&d_feedback);
}

JointMonitorByFeedback::~JointMonitorByFeedback(){
	dJointSetFeedback(GetJoint().Id(),NULL);
}


void JointMonitorByFeedback::GetForceAndTorque(dVector3 & force, dVector3 & torque){
	for(unsigned int i = 0; i < 3; ++i){
		force[i] = d_feedback.f2[i];// - d_feedback.f1[i];
		torque[i] = d_feedback.t2[i];// - d_feedback.t1[i];
	}
}


} /* namespace jointmanager */

} /* namespace modules */

} /* namespace plugin */

} /* namespace libwebots */

