/**
 * \file jointmonitor.h
 *
 *  \date Jan 24, 2013
 *  \author tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_MODULES_JOINTMONITOR_HH_
#define __LIBWEBOTS_PLUGIN_MODULES_JOINTMONITOR_HH_

#include <libwebots/plugin/modules/jointmanager/jointmanager.pb.h>
#include <plugins/physics.h>

namespace libwebots
{
namespace plugin {

namespace modules {

namespace jointmanager {

class Joint;

class JointMonitor {
public :
	typedef messages::jointmanager::JointDefinition::Frame Frame;
	JointMonitor(const Joint &,
	             Frame frame);
	virtual ~JointMonitor();

	void PrepareMessage(messages::jointmanager::Monitor & m);

protected :
	const Joint & GetJoint() const;

	virtual void GetForceAndTorque(dVector3 & force,dVector3 & torque) = 0;

private :
	typedef messages::jointmanager::JointDefinition JD;
	void ToBody(dBodyID body,
	            dVector3 & force,
	            dVector3 & torque);

	const Joint & d_joint;
	Frame         d_frame;

};

class JointMonitorByDifference : public JointMonitor{
public :
	JointMonitorByDifference(const Joint &,
	                         Frame frame);
	virtual ~JointMonitorByDifference();
protected :
	virtual void GetForceAndTorque(dVector3 & force,dVector3 & torque);
};


class JointMonitorByFeedback : public JointMonitor{
public :
	JointMonitorByFeedback(const Joint &,
	                        Frame frame);
	virtual ~JointMonitorByFeedback();
protected :
	virtual void GetForceAndTorque(dVector3 & force,dVector3 & torque);

private :
	dJointFeedback d_feedback;
};

} /* namespace jointmanager */

} /* namespace modules */

} /* namespace plugin */

} /* namespace libwebots */


#endif /* __LIBWEBOTS_PLUGIN_MODULES_JOINTMONITOR_HH_ */
