/**
 * \file ball.cc
 *
 *  \date Feb 8, 2013
 *  \author Alexandre Tuleu
 */

#include "ball.hh"
#include "../../../plugin.hh"

using namespace libwebots::plugin::modules::jointmanager;
using namespace libwebots::plugin::modules::messages::jointmanager;



Ball::Ball()
	: Joint()
//	, d_aMotorID(0)
{
}

void Ball::SetAnchor(const dVector3 worldPosition)
{
	if(!*this){
		return;
	}
	dJointSetBallAnchor(Id(),
	                    worldPosition[0],
	                    worldPosition[1],
	                    worldPosition[2]);

}

#include <iostream>

void Ball::SetAxis(const dVector3 worldAxis)
{
	if(!*this){
		return;
	}
	//assign parent vector to the one we need
	d_parentRef1 << worldAxis[0], worldAxis[1], worldAxis[2];
	d_parentRef1.normalize();

	Eigen::Vector3d origAlignedAxis(Eigen::Vector3d::Zero());
	if(d_parentRef1[1] == 0 && d_parentRef1[2] == 0){
		origAlignedAxis[1] = 1.0;
	} else {
		origAlignedAxis[0] = 1.0;
	}
	//the other refVector should be orthogonal to ref1 in world Coordiante;
	d_parentRef2 = d_parentRef1.cross(origAlignedAxis).normalized();
	//now translate all of them in body coordinates

	//safe becasue ODE only will modify 3 first values, and not the last one
	//(3D vector are 4 double arrays in ODE)
	dBodyVectorFromWorld(Child(),
	                     d_parentRef1[0],
	                     d_parentRef1[1],
	                     d_parentRef1[2],
	                     d_childRef1.data());
	dBodyVectorFromWorld(Child(),
	                     d_parentRef2[0],
	                     d_parentRef2[1],
	                     d_parentRef2[2],
	                     d_childRef2.data());

	dBodyVectorFromWorld(Parent(),
	                     d_parentRef1[0],
	                     d_parentRef1[1],
	                     d_parentRef1[2],
	                     d_parentRef1.data());

	dBodyVectorFromWorld(Parent(),
	                     d_parentRef2[0],
	                     d_parentRef2[1],
	                     d_parentRef2[2],
	                     d_parentRef2.data());




	d_childRef1.normalize();
	d_childRef2.normalize();
	d_parentRef1.normalize();
	d_parentRef2.normalize();

//	dJointSetAMotorAxis(d_aMotorID,0,0,
//	                    d_parentRef2[0],
//	                    d_parentRef2[1],
//	                    d_parentRef2[2]);
//
//	dJointSetAMotorAxis(d_aMotorID,0,0,
//	                    d_parentRef2[0],
//	                    d_parentRef2[1],
//	                    d_parentRef2[2]);
//
//	dJointSetAMotorAxis(d_aMotorID,1,0,
//	                    d_childRef1[0],
//	                    d_childRef1[1],
//	                    d_childRef1[2]);


	std::cerr << "Child 1: [" <<  d_childRef1.transpose() << "] 2:[" << d_childRef2.transpose()
	          << "] Parent 1:[" << d_parentRef1.transpose() << "] 2: ["<< d_parentRef2.transpose() << "]" << std::endl;

}


std::auto_ptr<Joint> Ball::Clone() const
{
	return std::auto_ptr<Joint>(new Ball(*this));
}

void Ball::SetParameter(int parameter,double value,unsigned int dim)
{
	if(!*this){return;}
	//dJointSetBallParam(d_aMotorID,parameter + dParamGroup * (dim-1),value);
	dJointSetBallParam(Id(),parameter + dParamGroup * (dim-1),value);
}

void Ball::AddEffort(const Eigen::Vector3d & value)
{
	//dJointAddAMotorTorques(d_aMotorID,value[0],value[1],0.0);
}



void Ball::ComputeCoordinateAndRate(double ts)
{
	//first find the rotation beween the two first axiss
	Eigen::Vector3d d_currentParentRef1,
	                d_currentChildRef1,
	                d_currentParentRef2,
	                d_currentChildRef2,
	                newPos(0.0,0.0,0.0);
	dBodyVectorToWorld(Parent(),d_parentRef1[0],d_parentRef1[1],d_parentRef1[2],d_currentParentRef1.data());
	dBodyVectorToWorld(Child(),d_childRef1[0],d_childRef1[1],d_childRef1[2],d_currentChildRef1.data());
	dBodyVectorToWorld(Parent(),d_parentRef2[0],d_parentRef2[1],d_parentRef2[2],d_currentParentRef2.data());
	dBodyVectorToWorld(Child(),d_childRef2[0],d_childRef2[1],d_childRef2[2],d_currentChildRef2.data());

	Eigen::Vector3d axis1(d_currentParentRef1.cross(d_currentChildRef1));

	if(axis1.isApproxToConstant(0.0,1e-5)){
		//first axis is 0
//		dJointSetAMotorAxis(d_aMotorID,0,0,
//		                    d_currentParentRef2[0],
//		                    d_currentParentRef2[1],
//		                    d_currentParentRef2[2]);
		axis1 = d_currentParentRef2;
	} else {
		double norm1 = axis1.norm();
		axis1 /= norm1;
//		dJointSetAMotorAxis(d_aMotorID,0,0,
//		                    axis1[0],axis1[1],axis1[2]);
		//now compute the torsion angle
		newPos[0] = std::asin(norm1);
		d_currentParentRef2 = Eigen::AngleAxis<double>(newPos[0],axis1) * d_currentParentRef2;
	}
	//second axis is the reference axis
	Eigen::Vector3d axis2(d_currentParentRef2.cross(d_currentChildRef2));
	if(axis2.isApproxToConstant(0.0,1e-5)){
//		dJointSetAMotorAxis(d_aMotorID,1,0,
//		                    d_currentChildRef1[0],
//		                    d_currentChildRef1[1],
//		                    d_currentChildRef1[2]);
		axis2 = d_currentChildRef1;
	}else {
		double norm2 = axis2.norm();
		axis2 /= norm2;
//		dJointSetAMotorAxis(d_aMotorID,1,0,
//		                    axis2[0],axis2[1],axis2[2]);
		newPos[1] = std::asin(norm2);
	}
	//if(ts == 0.0){
		GetCoordinateRate() = Eigen::Vector3d::Zero();
	//} else {
	//	GetCoordinateRate() = newPos - GetCoordinate();
	//	GetCoordinateRate() /= ts;
	//}
	GetCoordinate() = -newPos;
//	dJointSetAMotorAngle(d_aMotorID,0,newPos[0]);
//	dJointSetAMotorAngle(d_aMotorID,1,newPos[1]);
	d_axis1 = axis1;
	d_axis2 = axis2;

}
dJointID Ball::CreatePrimpl(dWorldID world, dJointGroupID group)
{
	if(*this) {return 0;}
//	d_aMotorID = dJointCreateAMotor(world,group);
//	dJointSetAMotorNumAxes(d_aMotorID,2);
	return dJointCreateBall(world,group);
}

void Ball::Attach(dBodyID parent, dBodyID child){
	if(!*this){
		return;
	}
	Joint::Attach(parent,child);
//	dJointAttach(d_aMotorID,parent,child);
}

unsigned int Ball::Dimension() const
{
	return 2;
}


void Ball::Draw(double size)
{
	dVector3 anchor;
	dJointGetBallAnchor(Id(),anchor);
	glDisable(GL_LIGHTING);
	glLineWidth(2.5);
	glColor4f(0,0,0,1.0);

	glBegin(GL_LINES);

	glVertex3f(anchor[0] + size,
			   anchor[1],
			   anchor[2]);

	glVertex3f(anchor[0] - size,
			   anchor[1],
			   anchor[2]);

	glEnd();

	glBegin(GL_LINES);

	glVertex3f(anchor[0],
			   anchor[1] + size,
			   anchor[2]);

	glVertex3f(anchor[0],
			   anchor[1] - size,
			   anchor[2]);

	glEnd();

	glBegin(GL_LINES);

	glVertex3f(anchor[0],
			   anchor[1],
			   anchor[2] + size);

	glVertex3f(anchor[0],
			   anchor[1],
			   anchor[2] - size);

	glEnd();
//	dJointGetAMotorAxis(d_aMotorID,0,axis1);
//	dJointGetAMotorAxis(d_aMotorID,1,axis2);

	glColor4f(1.0,0.0,0.0,1.0);

	glBegin(GL_LINES);
	glVertex3f(anchor[0] + d_axis1[0] * size,
	           anchor[1] + d_axis1[1] * size,
	           anchor[2] + d_axis1[2] * size);

	glVertex3f(anchor[0] - d_axis1[0] * size,
	           anchor[1] - d_axis1[1] * size,
	           anchor[2] - d_axis1[2] * size);
	glEnd();

	glColor4f(0.0,1.0,0.0,1.0);
	glBegin(GL_LINES);
	glVertex3f(anchor[0] + d_axis2[0] * size,
	           anchor[1] + d_axis2[1] * size,
	           anchor[2] + d_axis2[2]  * size);

	glVertex3f(anchor[0] - d_axis2[0] * size,
	           anchor[1] - d_axis2[1] * size,
	           anchor[2] - d_axis2[2] * size);
	glEnd();
	//std::cerr << Plugin::TimeStep() << " " << dJointGetAMotorAngle(d_aMotorID,0) << " " << dJointGetAMotorAngle(d_aMotorID,1) << std::endl;
}
