/**
 * \file ball.hh
 *
 *  \date Feb 8, 2013
 *  \autor Alexandre Tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_MODULES_JOINTCREATOR_JOINT_BALL_HH_
#define __LIBWEBOTS_PLUGIN_MODULES_JOINTCREATOR_JOINT_BALL_HH_


#include <libwebots/plugin/modules/jointmanager/joint.hh>

namespace libwebots
{
namespace plugin
{
namespace modules
{
namespace jointmanager
{
	/**
	 * A 3D dimensional Ball jointm, but with only two virtual dimension, as it
	 * implements conical angles. So Dimension() report 2 and AddEffort()
	 * GetCoordinate[Rate]() use 2D vector. This should be extended in the
	 * future with 3D euleur angle.
	 *
	 * In order to york, you must :
	 *  * provbide the anchor position
	 *  * the anchor axis
	 */
	class Ball : public Joint
	{
	public :
		Ball();
		virtual unsigned int Dimension() const;
		virtual void SetAnchor(const dVector3 worldPosition);
		virtual void SetAxis(const dVector3 worldAxis);
		virtual std::auto_ptr<Joint> Clone() const;
		virtual void SetParameter(int parameter,
		                          double value,
		                          unsigned int dimension);
		virtual void AddEffort(const Eigen::Vector3d & value);
		virtual void ComputeCoordinateAndRate(double ts);
		virtual dJointID CreatePrimpl(dWorldID world, dJointGroupID group);
		virtual void Attach(dBodyID parent, dBodyID child);
		virtual void Draw(double size);
		EIGEN_MAKE_ALIGNED_OPERATOR_NEW_IF_VECTORIZABLE_FIXED_SIZE(double,3)
	private :
		//dJointID d_aMotorID;
		Eigen::Vector3d d_parentRef1,d_parentRef2,d_childRef1,d_childRef2;
		Eigen::Vector3d d_axis1,d_axis2;
	};

} //namespace jointmanager
} //namespace modules
} //namespace plugin
} //namespace libwebots


#endif //__LIBWEBOTS_PLUGIN_MODULES_JOINTCREATOR_JOINT_BALL_HH_
