/**
 * \file hinge.cc
 *
 *  \date Feb 8, 2013
 *  \author Alexandre Tuleu
 */

#include "hinge.hh"


using namespace libwebots::plugin::modules::jointmanager;
using namespace libwebots::plugin::modules::messages::jointmanager;


void Hinge::SetAnchor(const dVector3 worldPosition)
{
	if(!*this) { return; }
	dJointSetHingeAnchor(Id(),
	                      worldPosition[0],
	                      worldPosition[1],
	                      worldPosition[2]);
}



void Hinge::SetAxis(const dVector3 worldAxis)
{
	if(!*this) { return; }
	dJointSetHingeAxis(Id(),
	                   worldAxis[0],
	                   worldAxis[1],
	                   worldAxis[2]);
}

unsigned int Hinge::Dimension() const {
	return 1;
}

std::auto_ptr<Joint> Hinge::Clone() const
{
	return std::auto_ptr<Joint>(new Hinge(*this));
}



void Hinge::SetParameter(int parameter, double value,unsigned int)
{
	if(!*this) { return; }
	dJointSetHingeParam(Id(),parameter,value);
}



void Hinge::AddEffort(const Eigen::Vector3d & value)
{
	if(!*this) { return; }
	dJointAddHingeTorque(Id(),-value.x());
}



void Hinge::ComputeCoordinateAndRate(double)
{
	if(!*this) { return; }
	GetCoordinate()[0] = -dJointGetHingeAngle(Id());
	GetCoordinateRate()[0] = -dJointGetHingeAngleRate(Id());

}



dJointID Hinge::CreatePrimpl(dWorldID world, dJointGroupID group)
{
	if(*this) { return 0; }
	return dJointCreateHinge(world,group);
}

void Hinge::Draw(double size){
	dVector3 pos,axis;
	dJointGetHingeAnchor(Id(),pos);
	dJointGetHingeAxis(Id(),axis);
	glLineWidth(2.5);
	glColor3f(0.0,0.0,0.0);
	glBegin(GL_LINES);

	glVertex3d(pos[0] + size * axis[0],
	           pos[1] + size * axis[1],
	           pos[2] + size * axis[2]);

	glVertex3d(pos[0] - size * axis[0],
	           pos[1] - size * axis[1],
	           pos[2] - size * axis[2]);

	glEnd();
}

