/**
 * \file passivedynamics.hh
 *
 *  \date Nov 24, 2011
 *  \author Alexandre Tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_MODULES_JOINTMANAGER_PASSIVEDYNAMICS_HH_
#define __LIBWEBOTS_PLUGIN_MODULES_JOINTMANAGER_PASSIVEDYNAMICS_HH_

#include <libwebots/plugin/modules/jointmanager/jointmanager.pb.h>
#include "joints/joints.hh"
#include <tr1/memory>

namespace libwebots
{
namespace plugin
{
namespace modules
{
namespace jointmanager
{
	class PassiveDynamics
	{
	public :
		friend std::ostream & operator << (std::ostream & out , const PassiveDynamics & pd);

		PassiveDynamics(std::tr1::shared_ptr<Joint> & associatedJoint);
		virtual ~PassiveDynamics();

		void SetFromMessage(messages::jointmanager::SetPassiveDynamics const & message);

		void Step() const;

	private:
		typedef ::google::protobuf::RepeatedPtrField< ::libwebots::plugin::modules::messages::jointmanager::PassiveDynamic> ListOfPassiveDynamics;
		struct Dynamics {
			Eigen::Array3d stiffness;
			Eigen::Array3d damping;
			Eigen::Array3d precompression;
			Dynamics() : stiffness(0,0,0),damping(0,0,0),precompression(0,0,0){}
			EIGEN_MAKE_ALIGNED_OPERATOR_NEW_IF_VECTORIZABLE_FIXED_SIZE(double,3);
		};

		void SetDynamicFromMessage(const ListOfPassiveDynamics & data,
		                           Dynamics & dynamics);

		void SetScalarDynamicFromMessage(messages::jointmanager::PassiveDynamic const &message,
		                                 Dynamics & dynamic);

		std::tr1::shared_ptr<Joint> d_joint;

		Dynamics d_dynamic;
		Dynamics d_negativeDynamic;

		Eigen::Array3d d_referencePosition;

		bool d_useNegative;
	};

	std::ostream & operator << (std::ostream & out , const PassiveDynamics & pd);
}
}
}
}


#endif // __LIBWEBOTS_PLUGIN_MODULES_JOINTMANAGER_PASSIVEDYNAMICS_HH_
