/**
 * \file profiles.cc
 *
 *  \date Mar 2, 2011
 *  \author Alexandre Tuleu
 */

#include "profiles.hh"
#include <cmath>

using namespace libwebots::plugin::modules::push;

void libwebots::plugin::modules::Push::RegisterProfiles()
{
	PushAction::RegisterProfile(messages::push::Profile::Step,
	                            new push::StepProfile());

	PushAction::RegisterProfile(messages::push::Profile::Linear,
	                            new push::LinearProfile());

	PushAction::RegisterProfile(messages::push::Profile::Quadratic,
	                            new push::QuadraticProfile());
}


StepProfile::StepProfile()
{
}

StepProfile::~StepProfile()
{
}

std::auto_ptr<Profile> StepProfile::Clone() const
{
	return std::auto_ptr<Profile>(new StepProfile(*this));
}

double StepProfile::ComputeValue(double ratioDone)
{
	if (ratioDone == 0.0 || ratioDone == 1.0)
	{
		return 0.0;
	}

	return 1.0;
}

LinearProfile::LinearProfile()
{
}

LinearProfile::~LinearProfile()
{
}

std::auto_ptr<Profile> LinearProfile::Clone() const
{
	return std::auto_ptr<Profile>(new LinearProfile(*this));
}

double LinearProfile::ComputeValue(double ratioDone)
{
	if (ratioDone <= 0.1)
	{
		return ratioDone / 0.1;
	}

	if (ratioDone >= 0.9)
	{
		return 1.0 - (ratioDone - 0.9) / 0.1;
	}

	return 1.0;
}

QuadraticProfile::QuadraticProfile()
{
}

QuadraticProfile::~QuadraticProfile()
{
}

std::auto_ptr<Profile> QuadraticProfile::Clone() const
{
	return std::auto_ptr<Profile>(new QuadraticProfile(*this));
}

double QuadraticProfile::ComputeValue(double ratioDone)
{
	if (ratioDone <= 0.05)
	{
		return std::pow(ratioDone, 2) * 200;
	}
	else if (ratioDone <= 0.1)
	{
		return 1.0 - std::pow(0.1 - ratioDone, 2.0) * 200;
	}
	else if (ratioDone <= 0.9)
	{
		return 1.0;
	}
	else if (ratioDone <= 0.95)
	{
		return 1.0 - std::pow(ratioDone - 0.9, 2) * 200;
	}
	else
	{
		return std::pow(1.0 - ratioDone, 2) * 200;
	}
}
