/**
 * \file push.hh
 *
 *  \date Mar 2, 2011
 *  \author Alexandre Tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_MODULES_PUSH_H__
#define __LIBWEBOTS_PLUGIN_MODULES_PUSH_H__

#include <libwebots/plugin/modules/push/profile.hh>
#include <libwebots/plugin/physicsmodule.hh>
#include <libwebots/plugin/modules/debug/debug.hh>
#include <libwebots/plugin/modules/push/push.pb.h>
#include <memory>
#include <map>
#include <list>

namespace libwebots
{
namespace plugin
{
namespace modules
{
	class Push : public PhysicsModule
	{
		public:
			Push(dWorldID world,
			     dSpaceID space,
			     dJointGroupID jointGroup,
			     Module::MapByTypeID const &dependencies);
			virtual ~Push();

			virtual void Step();
			virtual void Draw();

			static void RegisterDependencies();

			//register profiles here.
			static void RegisterProfiles();
		private:
			class PushAction
			{
				private:
					PushAction(dBodyID id,
					           double x,
					           double y,
					           double z,
					           double tx,
					           double ty,
					           double tz,
					           double duration,
					           std::auto_ptr<push::Profile> profile,
					           double currentTime);
				public:
					virtual ~PushAction();

					void Apply();

					void Draw();

					bool Done() const;

					static void RegisterProfile(const messages::push::Profile::Values id,
					                            push::Profile *profile);
					static void SetTime(double time);

					static PushAction* Create(messages::push::Push const &message,
					                          double currentTime,
					                          interfaces::Interface<Debug> &debugOutput);

				private:
					typedef std::map<messages::push::Profile::Values, push::Profile *> ProfileFactory;

					static ProfileFactory s_profiles;
					static double s_time;

					double GetRatio() const;

					dBodyID d_body;
					double d_target[6];
					double d_current[6];
					double d_duration;
					double d_startTime;
					double d_currentProfileRatio;
					std::auto_ptr<push::Profile> d_profile;
			};

			void Handler(messages::push::Push const & message);
			std::list<PushAction *> d_pushes;
			interfaces::Interface<Debug> d_debug;
	};
}
}
}

#endif //__LIBWEBOTS_PLUGIN_MODULES_PUSH_H__
