#ifndef __LIBWEBOTS_PLUGIN_MODULES_STATE_H__
#define __LIBWEBOTS_PLUGIN_MODULES_STATE_H__

#include <libwebots/plugin/physicsmodule.hh>
#include <libwebots/plugin/state.hh>
#include <libwebots/plugin/utils.hh>
#include <libwebots/plugin/modules/state/state.pb.h>
#include <libwebots/plugin/modules/debug/debug.hh>
#include <Eigen/Geometry>

#include <vector>
#include <map>

namespace libwebots
{
namespace plugin
{
namespace modules
{
	class State : public PhysicsModule , public interfaces::Debugable
	{
		struct Monitor
		{
			dBodyID Body;

			double PreviousAngularVelocity[3];
			double PreviousLinearVelocity[3];
		};

		plugin::State d_state;
		std::map<std::string, Monitor *> d_monitors;
		double d_previousTime;

		std::map<dBodyID, std::string> d_bodyMap;
		std::vector<std::string> d_names;

		std::vector<dBodyID> d_bodyDump;
		std::vector<dJointID> d_jointDump;


		public:
			State(dWorldID world,
			      dSpaceID space,
			      dJointGroupID jointGroup,
			      Module::MapByTypeID const &dependencies);

			~State();
			virtual void StepEnd();

			void AddMonitor(std::string const &name);
		private:
			void Handler(messages::state::State const &message);

			void SetState(std::string const &name, messages::state::SetState const &message);
			void RegisterNames(messages::state::RegisterNames const &message);

			dBodyID FindParent(dBodyID body, dVector3 axis);

			void Dump(std::string const &body);

			void Dump();
			void Dump(std::vector<dBodyID> const &bodies,
			          std::vector<dJointID> const &joints);

			void Dump(messages::state::Dump &dump,
			          dBodyID body);

			void Dump(messages::state::Dump &dump,
			          dJointID joint,
			          std::map<dBodyID, size_t> const &bodyIndex);

			dBodyID CorrectedBody(dBodyID body);

			void InitializeDump();

			void InitialState(messages::state::InitialState const &message);
			std::vector<Body> FindStateTrees(messages::state::InitialState const &message,
			                                 std::map<dBodyID, messages::state::InitialState::Description> &defs);

			void InitialState(Body &body,
			                  std::map<dBodyID, messages::state::InitialState::Description> const &defs,
			                  Eigen::Isometry3d const &transform);
	};
}
}
}

#endif /* __LIBWEBOTS_PLUGIN_MODULES_STATE_H__ */

