#include "statebuilder.hh"
#include <iostream>

using namespace libwebots::plugin::modules::messages;
using namespace libwebots::communication;
using namespace std;

builders::State::InitialStateDefinition::InitialStateDefinition(std::string const &name,
                                                                double angle,
                                                                double angularVelocity,
                                                                double linvelx,
                                                                double linvely,
                                                                double linvelz)
:
	Name(name),
	Angle(angle),
	AngularVelocity(angularVelocity)
{
	LinearVelocity[0] = linvelx;
	LinearVelocity[1] = linvely;
	LinearVelocity[2] = linvelz;
}

Message
builders::State::Push()
{
	return Create(state::Action::Push);
}

Message
builders::State::Pop()
{
	return Create(state::Action::Pop);
}

Message
builders::State::Apply()
{
	return Create(state::Action::Apply);
}

Message
builders::State::Update()
{
	return Create(state::Action::Update);
}

Message
builders::State::RegisterBody(string const &body)
{
	state::State st;

	st.set_action(state::Action::RegisterBody);
	st.set_body(body);

	return Messages::Create(state::state, st);
}

Message
builders::State::MonitorBody(string const &body)
{
	state::State st;

	st.set_action(state::Action::MonitorBody);
	st.set_body(body);

	return Messages::Create(state::state, st);
}

Message
builders::State::SetPosition(string const &body,
                             double x,
                             double y,
                             double z)
{
	state::State st;

	st.set_action(state::Action::Set);
	st.set_body(body);

	state::SetState &set = *(st.mutable_setstate());

	set.add_position(x);
	set.add_position(y);
	set.add_position(z);

	return Messages::Create(state::state, st);
}

Message
builders::State::SetRotation(string const &body,
                             double w,
                             double x,
                             double y,
                             double z)
{
	state::State st;

	st.set_action(state::Action::Set);
	st.set_body(body);

	state::SetState &set = *(st.mutable_setstate());

	set.add_rotation(w);
	set.add_rotation(x);
	set.add_rotation(y);
	set.add_rotation(z);

	return Messages::Create(state::state, st);
}

Message
builders::State::SetLinearVelocity(string const &body,
                                   double x,
                                   double y,
                                   double z)
{
	state::State st;

	st.set_action(state::Action::Set);
	st.set_body(body);

	state::SetState &set = *(st.mutable_setstate());

	set.add_linear_velocity(x);
	set.add_linear_velocity(y);
	set.add_linear_velocity(z);

	return Messages::Create(state::state, st);
}

Message
builders::State::SetAngularVelocity(string const &body,
                                    double x,
                                    double y,
                                    double z)
{
	state::State st;

	st.set_action(state::Action::Set);
	st.set_body(body);

	state::SetState &set = *(st.mutable_setstate());

	set.add_angular_velocity(x);
	set.add_angular_velocity(y);
	set.add_angular_velocity(z);

	return Messages::Create(state::state, st);
}

Message
builders::State::Create(state::Action::Values action)
{
	state::State st;

	st.set_action(action);

	return Messages::Create(state::state, st);
}

Message
builders::State::RegisterNames(vector<string> const &names)
{
	state::State st;

	st.set_action(state::Action::RegisterNames);

	modules::messages::state::RegisterNames &r = *(st.mutable_registernames());

	for (vector<string>::const_iterator iter = names.begin(); iter != names.end(); ++iter)
	{
		r.add_names(*iter);
	}

	return Messages::Create(state::state, st);
}

Message
builders::State::Dump()
{
	return Create(state::Action::Dump);
}

Message
builders::State::InitialState(vector<InitialStateDefinition> const &states)
{
	vector<InitialStateDefinition>::const_iterator iter;

	state::State st;

	st.set_action(state::Action::InitialState);

	modules::messages::state::InitialState &sst = *(st.mutable_initialstate());

	for (iter = states.begin(); iter != states.end(); ++iter)
	{
		modules::messages::state::InitialState::Description *desc;

		desc = sst.add_joints();

		desc->set_name(iter->Name);
		desc->set_angle(iter->Angle);
		desc->set_angularvelocity(iter->AngularVelocity);

		for (size_t i = 0; i < 3; ++i)
		{
			desc->add_linearvelocity(iter->LinearVelocity[i]);
		}
	}

	return Messages::Create(state::state, st);
}

Message
builders::State::InitialState(InitialStateDefinition const &definition)
{
	vector<InitialStateDefinition> defs;

	defs.push_back(definition);

	return InitialState(defs);
}

Message
builders::State::InitialState(string const &name,
                              double angle,
                              double angularVelocity,
                              double linearVelocity)
{
	return InitialState(InitialStateDefinition(name, angle, angularVelocity, linearVelocity));
}
