#ifndef __LIBWEBOTS_PLUGIN_MODULES_MESSAGES_BUILDERS_STATE_H__
#define __LIBWEBOTS_PLUGIN_MODULES_MESSAGES_BUILDERS_STATE_H__

#include <libwebots/communication/messages.hh>
#include <libwebots/plugin/modules/state/state.pb.h>
#include <string>
#include <vector>

namespace libwebots
{
namespace plugin
{
namespace modules
{
namespace messages
{
namespace builders
{
	class State
	{
		public:
			struct InitialStateDefinition
			{
				std::string Name;
				double Angle;
				double AngularVelocity;
				double LinearVelocity[3];

				InitialStateDefinition(std::string const &name,
				                       double angle,
				                       double angularVelocity = 0,
				                       double linvelx = 0,
				                       double linvely = 0,
				                       double linvelz = 0);
			};

			static communication::Message Push();
			static communication::Message Pop();
			static communication::Message Apply();
			static communication::Message Update();

			static communication::Message RegisterBody(std::string const &body);
			static communication::Message MonitorBody(std::string const &body);

			static communication::Message SetPosition(std::string const &body,
			                                          double x,
			                                          double y,
			                                          double z);

			static communication::Message SetRotation(std::string const &body,
			                                          double w,
			                                          double x,
			                                          double y,
			                                          double z);

			static communication::Message SetLinearVelocity(std::string const &body,
			                                                double x,
			                                                double y,
			                                                double z);

			static communication::Message SetAngularVelocity(std::string const &body,
			                                                 double x,
			                                                 double y,
			                                                 double z);

			static communication::Message RegisterNames(std::vector<std::string> const &names);
			static communication::Message Dump();

			static communication::Message InitialState(std::vector<InitialStateDefinition> const &states);
			static communication::Message InitialState(InitialStateDefinition const &definition);

			static communication::Message InitialState(std::string const &name,
			                                           double angle,
			                                           double angularVelocity = 0,
			                                           double linearVelocity = 0);
		private:
			static communication::Message Create(state::Action::Values action);
	};
}
}
}
}
}

#endif /* __LIBWEBOTS_PLUGIN_MODULES_MESSAGES_BUILDERS_STATE_H__ */

