/**
 * \file physicsmodule.cc
 *
 *  \date Mar 8, 2011
 *  \author Alexandre Tuleu
 */


#include "physicsmodule.hh"

using namespace libwebots::plugin;

/**
 * @class libwebots::plugin::PhysicsModule
 * @brief Physics module base class
 *
 * This module base class can be subclassed when implementing a module that
 * needs to hook into the various webots callbacks (Step(), StepEnd(),
 * Collide(), etc). The physics module implements the WebotsInterface class,
 * which defines the virtual methods that can be overriden. These methods
 * map exactly to the conventional webots plugin callbacks.
 *
 * @see Module class for more information on registering module dependencies
 *
 **/

/**
 * @brief Create a new physics module
 * @param world the world id
 * @param space the space id
 * @param jointGroup the joint group id
 * @param dependencies the registered dependency map
 *
 * Create a new physics module.
 *
 **/
PhysicsModule::PhysicsModule(dWorldID world,
                             dSpaceID space,
                             dJointGroupID jointGroup,
                             Module::MapByTypeID const &dependencies)
:
	Module(dependencies),
	WebotsInterface(world, space, jointGroup)
{

}

PhysicsModule::~PhysicsModule()
{
}
