#ifndef __LIBWEBOTS_PLUGIN_STATE_H__
#define __LIBWEBOTS_PLUGIN_STATE_H__

#include <plugins/physics.h>
#include <vector>
#include <stack>
#include <string>

namespace libwebots
{
namespace plugin
{
	class State
	{
		public:
			struct Item
			{
				dBodyID Body;

				dReal Position[3];
				dQuaternion Rotation;

				dReal LinearVelocity[3];
				dReal AngularVelocity[3];

				Item(dBodyID body);

				void Reset();
				void Store();
			};

			State();
			State(dBodyID body);
			State(dSpaceID space);

			void Push();
			void Pop();
			void Apply();
			void Update();

			std::vector<dBodyID> Bodies();
			std::vector<dBodyID> const &Bodies() const;

			std::vector<Item *> Top();
			std::vector<Item *> const &Top() const;

			void Register(dBodyID body);
			void Register(dSpaceID space);
			void Register(std::string const &bodyDef);
		private:
			std::vector<dBodyID> d_bodies;
			std::stack<std::vector<Item *> > d_state;

			bool BodyExists(dBodyID body) const;
			void ResolveJoints(dBodyID body);
	};
}
}

#endif /* __LIBWEBOTS_PLUGIN_STATE_H__ */

