/**
 * \file typemap.hh
 *
 *  \date Mar 7, 2011
 *  \author Alexandre Tuleu
 */

#ifndef __LIBWEBOTS_PLUGIN_TYPEMAP_HH_
#define __LIBWEBOTS_PLUGIN_TYPEMAP_HH_

namespace libwebots
{
namespace plugin
{
	/**
	 * @brief Type map base class
	 *
	 * Base class for type ids
	 *
	 **/
	class TypeMapBase
	{
	};

	/**
	 * @brief Template type singleton identifier class
	 *
	 * Template class implementing a singleton providing an identifier
	 * (by pointer) to a particular type. This can then be used to compare
	 * types (by comparing TypeMap<T>::ID()).
	 *
	 **/
	template <typename T>
	class TypeMap : public TypeMapBase
	{
		static TypeMapBase *s_instance;

		public:
			static TypeMapBase *ID();

	};

	/**
	 * @brief Type id
	 *
	 * A type identifier
	 *
	 **/
	typedef TypeMapBase *TypeID;

	template <typename T>
	TypeMapBase *TypeMap<T>::s_instance(0);

	/**
	 * @brief Type id for type
	 *
	 * Get a TypeID for a given type T
	 *
	 **/
	template <typename T>
	TypeID
	TypeMap<T>::ID()
	{
		if (!s_instance)
		{
			s_instance = new TypeMapBase();
		}

		return s_instance;
	}
}
}

#endif // __LIBWEBOTS_PLUGIN_TYPEMAP_HH_
