#include "utils.hh"
#include <algorithm>

using namespace libwebots::plugin;
using namespace std;

dJointID
Utils::FindWebotsJoint(dBodyID body, dBodyID &parent, dVector3 axis)
{
	int num = dBodyGetNumJoints(body);

	for (int i = 0; i < num; ++i)
	{
		dJointID joint = dBodyGetJoint(body, i);
		int type = dJointGetType(joint);

		if (dJointGetBody(joint, 1) != body ||
		    (type != dJointTypeHinge && type != dJointTypeSlider))
		{
			continue;
		}

		if (type == dJointTypeHinge)
		{
			dJointGetHingeAxis(joint, axis);
		}
		else
		{
			dJointGetSliderAxis(joint, axis);
		}

		parent = dJointGetBody(joint, 0);
		return joint;
	}

	return 0;
}

dJointID
Utils::FindWebotsJoint(dBodyID body, dVector3 axis)
{
	dBodyID parent;

	return FindWebotsJoint(body, parent, axis);
}

dJointID
Utils::FindWebotsJoint(dBodyID body, dBodyID &parent)
{
	dVector3 axis;
	return FindWebotsJoint(body, parent, axis);
}

dJointID
Utils::FindWebotsJoint(dBodyID body)
{
	dBodyID parent;
	dVector3 axis;

	return FindWebotsJoint(body, parent, axis);
}

vector<dBodyID>
Utils::AllBodies(dBodyID origin)
{
	vector<dBodyID> bodies;

	AllBodies(origin, 0, bodies);
	return bodies;
}

void
Utils::AllBodies(dBodyID body, dBodyID origin, vector<dBodyID> &bodies)
{
	if (!body)
	{
		return;
	}

	if (find(bodies.begin(), bodies.end(), body) != bodies.end())
	{
		return;
	}

	bodies.push_back(body);
	int num = dBodyGetNumJoints(body);

	for (int i = 0; i < num; ++i)
	{
		dJointID joint = dBodyGetJoint(body, i);

		dBodyID con0 = dJointGetBody(joint, 0);
		dBodyID con1 = dJointGetBody(joint, 1);

		dBodyID other = con0 == body ? con1 : con0;

		if (other == origin)
		{
			continue;
		}

		AllBodies(con0 == body ? con1 : con0, body, bodies);
	}
}

vector<dJointID>
Utils::AllJoints(dBodyID origin)
{
	vector<dJointID> joints;

	AllJoints(origin, 0, joints);
	return joints;
}

void
Utils::AllJoints(dBodyID body, dBodyID origin, vector<dJointID> &joints)
{
	if (!body)
	{
		return;
	}

	int num = dBodyGetNumJoints(body);

	for (int i = 0; i < num; ++i)
	{
		dJointID joint = dBodyGetJoint(body, i);

		if (find(joints.begin(), joints.end(), joint) != joints.end())
		{
			continue;
		}

		joints.push_back(joint);

		dBodyID con0 = dJointGetBody(joint, 0);
		dBodyID con1 = dJointGetBody(joint, 1);

		dBodyID other = con0 == body ? con1 : con0;

		if (other == origin)
		{
			continue;
		}

		AllJoints(con0 == body ? con1 : con0, body, joints);
	}
}
