#ifndef __LIBWEBOTS_TOOLS_PROTODOC_COMMAND_LINE_H__
#define __LIBWEBOTS_TOOLS_PROTODOC_COMMAND_LINE_H__

#include <string>
#include <vector>
#include <map>

namespace libwebots
{
namespace tools
{
namespace protodoc
{
	class CommandLine
	{
		static CommandLine *s_instance;

		std::vector<std::string> d_arguments;
		size_t d_numArgs;
		std::string d_name;
		std::string d_rest;

		public:
			struct Option
			{
				std::string Name;
				char Abbrev;
				std::string Description;
				bool Value;

				Option(std::string const &name,
				       char abbrev = '\0',
				       std::string const &description = "",
				       bool defaultValue = false);

				virtual ~Option();

				operator bool() const;
			};

			struct OptionArg : public Option
			{
				std::string Argument;
				std::vector<std::string> Arguments;

				OptionArg(std::string const &name,
				          char abbrev = '\0',
				          std::string const &description = "",
				          std::string const &defaultValue = "");
			};

			static CommandLine &Instance();
			static CommandLine &Initialize(std::string const &name, std::string const &rest = "");

			CommandLine(std::string const &name = "", std::string const &rest = "");
			~CommandLine();

			template <typename T>
			T &Add(T const &option);

			void Parse(int argc, char * const *argv);

			std::vector<std::string> const &Arguments() const;

			template <typename T>
			T &GetOption(std::string const &name);

			template <typename T>
			T &GetOption(char abbrev);

			void ShowHelp();
		private:
			std::vector<Option *> d_options;
			std::map<char, Option *> d_optionAbbrevMap;
			std::map<std::string, Option *> d_optionNameMap;
	};

	template <typename T>
	T &CommandLine::GetOption(std::string const &name)
	{
		return dynamic_cast<T &>(*d_optionNameMap[name]);
	}

	template <typename T>
	T &CommandLine::GetOption(char abbrev)
	{
		return dynamic_cast<T &>(*d_optionAbbrevMap[abbrev]);
	}

	template <typename T>
	T &CommandLine::Add(T const &option)
	{
		T *opt;

		opt = new T(option);
		d_options.push_back(opt);

		if (opt->Name != "")
		{
			++d_numArgs;
			d_optionNameMap[opt->Name] = opt;
		}

		if (opt->Abbrev)
		{
			d_optionAbbrevMap[opt->Abbrev] = opt;
		}

		return *opt;
	}
}
}
}

#endif /* __LIBWEBOTS_TOOLS_PROTODOC_COMMAND_LINE_H__ */
