#ifndef __LIBWEBOTS_TOOLS_PROTODOC_DOC_FINDER_H__
#define __LIBWEBOTS_TOOLS_PROTODOC_DOC_FINDER_H__

#include <google/protobuf/descriptor.h>
#include <google/protobuf/descriptor.pb.h>
#include <string>
#include <istream>
#include <map>
#include <stack>

namespace libwebots
{
namespace tools
{
namespace protodoc
{
	class DocFinder
	{
		struct Documentation
		{
			google::protobuf::SourceCodeInfo::Location Location;
			std::string Comment;
		};

		std::string d_filename;
		std::map<google::protobuf::Message const *, Documentation> d_locations;
		std::vector<std::string> d_lines;

		public:
			DocFinder();
			DocFinder(std::string const &filename,
			          google::protobuf::FileDescriptorProto const &proto);

			bool Lookup(google::protobuf::Message const &message,
			            std::string &comment);

			std::string Lookup(google::protobuf::Message const &message);
		private:
			void AddLocation(google::protobuf::FileDescriptorProto const &proto,
			                 google::protobuf::Message const &message,
			                 std::deque<int> const &path);

			std::string ExtractDocumentation(google::protobuf::SourceCodeInfo::Location const &location);

			bool FindPath(google::protobuf::FileDescriptorProto const &proto,
			              std::deque<int>                       const &path,
			              google::protobuf::SourceCodeInfo::Location  &location);

			bool LocationMatch(google::protobuf::SourceCodeInfo::Location const &location,
			                   std::deque<int>                            const &path);

			void Scan(google::protobuf::FileDescriptorProto const &proto);

			void Scan(google::protobuf::FileDescriptorProto const &proto,
			          google::protobuf::DescriptorProto     const &message,
			          std::deque<int>                             &path);

			void Scan(google::protobuf::FileDescriptorProto     const &proto,
			          google::protobuf::EnumDescriptorProto     const &en,
			          std::deque<int>                                 &path);

			void Scan(google::protobuf::FileDescriptorProto      const &proto,
			          google::protobuf::FieldDescriptorProto     const &field,
			          std::deque<int>                                  &path);
	};
}
}
}

#endif /* __LIBWEBOTS_TOOLS_PROTODOC_DOC_FINDER_H__ */

