#ifndef __LIBWEBOTS_TOOLS_PROTODOC_DOXYGEN_H__
#define __LIBWEBOTS_TOOLS_PROTODOC_DOXYGEN_H__

#include <iosfwd>
#include <string>
#include <vector>
#include <utility>

namespace libwebots
{
namespace tools
{
namespace protodoc
{
	class Doxygen
	{
		std::string d_description;
		std::string d_extras;
		std::string d_brief;
		std::vector<std::pair<std::string, std::string> > d_params;
		std::string d_returns;
		std::string d_className;
		std::string d_classInclude;

		public:
			Doxygen(std::string const &description,
			        std::string const &extras = "");

			Doxygen &Brief(std::string const &description);
			Doxygen &Param(std::string const &name,
			               std::string const &description);
			Doxygen &Returns(std::string const &description);
			Doxygen &Class(std::string const &name);

			std::string Generate() const;

			std::string NormalizeDescription() const;

			template <typename T>
			void RemoveEmptyLines(std::vector<std::string> &lines,
			                      T begin,
			                      T end) const;

			void DoRemoveEmptyLines(std::vector<std::string> &lines,
			                        std::vector<std::string>::iterator begin,
			                        std::vector<std::string>::iterator end) const;

			void DoRemoveEmptyLines(std::vector<std::string> &lines,
			                        std::vector<std::string>::reverse_iterator begin,
			                        std::vector<std::string>::reverse_iterator end) const;
	};

	template <typename T>
	void
	Doxygen::RemoveEmptyLines(std::vector<std::string> &lines,
	                          T begin,
	                          T end) const
	{
		T iter = begin;
		T prev = begin;

		while (iter != end)
		{
			if (!iter->empty())
			{
				DoRemoveEmptyLines(lines, prev, iter);
				return;
			}

			++iter;
		}

		DoRemoveEmptyLines(lines, begin, end);
	}
}
}
}

std::ostream &operator<<(std::ostream &stream, libwebots::tools::protodoc::Doxygen const &doxygen);

#endif /* __LIBWEBOTS_TOOLS_PROTODOC_DOXYGEN_H__ */

