#ifndef __LIBWEBOTS_TOOLS_PROTODOC_GENERATOR_H__
#define __LIBWEBOTS_TOOLS_PROTODOC_GENERATOR_H__

#include <google/protobuf/descriptor.h>
#include <string>
#include <iosfwd>
#include <vector>

#include "docfinder.hh"
#include "indentstream.hh"

namespace libwebots
{
namespace tools
{
namespace protodoc
{
	class Generator
	{
		std::string d_filename;
		google::protobuf::FileDescriptorProto const &d_proto;
		DocFinder d_docFinder;
		size_t d_indent;

		public:
			Generator(std::string const &filename,
			          google::protobuf::FileDescriptorProto const &proto);

			void Write(std::ostream &output);
		private:
			void Write(IndentStream &output,
			           google::protobuf::DescriptorProto const &message);

			void Write(IndentStream &output,
			           google::protobuf::EnumDescriptorProto const &en);

			void Write(IndentStream &output,
			           google::protobuf::EnumValueDescriptorProto const &val);

			void Write(IndentStream &output,
			           google::protobuf::FieldDescriptorProto const &field);

			std::vector<std::string> SplitPackage(std::string const &package);

			void WriteComment(IndentStream &output,
			                  google::protobuf::Message const &message);

			void WriteComment(IndentStream &output,
			                  google::protobuf::DescriptorProto const &message);

			std::string ClassName(std::string const &name) const;
	};
}
}
}

#endif /* __LIBWEBOTS_TOOLS_PROTODOC_GENERATOR_H__ */

